@@ -4,6 +4,7 @@ import contextlib
 from collections import OrderedDict
 from urllib.parse import urljoin
 from html.parser import HTMLParser
+from packaging.version import parse as parse_version
 
 from pylibimport.utils import get_name_version, EXTENSIONS
 
@@ -30,16 +31,20 @@ class HttpListVersions(HTMLParser):
     Args:
         index_url (str) ['https://pypi.org/simple/']: Simple url to get the package and it's versions from.
         extensions (list/str) [None]: List of allowed extensions (Example: [".whl", ".tar.gz"]).
+        min_version (str)[None]: Minimum version to allow.
+        exclude (list)[None]: List of versions that are excluded.
     """
 
     EXTENSIONS = EXTENSIONS
 
-    def __init__(self, index_url='https://pypi.org/simple/', extensions=None, **kwargs):
+    def __init__(self, index_url='https://pypi.org/simple/', extensions=None, min_version=None, exclude=None, **kwargs):
         """Simple HTML parser to get the plugin, url names.
 
         Args:
             index_url (str) ['https://pypi.org/simple/']: Simple url to get the package and it's versions from.
             extensions (list/str) [None]: List of allowed extensions (Example: [".whl", ".tar.gz"]).
+            min_version (str)[None]: Minimum version to allow.
+            exclude (list)[None]: List of versions that are excluded.
         """
         if extensions is None:
             extensions = self.EXTENSIONS
@@ -49,11 +54,45 @@ class HttpListVersions(HTMLParser):
         if index_url and not index_url.endswith('/'):
             index_url += '/'
 
+        if exclude is None:
+            exclude = []
+        elif not isinstance(exclude, (list, tuple)):
+            exclude = [exclude]
+        exclude = list(exclude)
+
         self.index_url = index_url
         self.extensions = extensions
+        self._min_version = min_version
+        self._min_version_parsed = parse_version(min_version or '-999.-999.-999')
+        self._exclude = exclude
         self.saved_data = OrderedDict()
         super().__init__(**kwargs)
 
+    @property
+    def min_version(self):
+        return self._min_version
+
+    @min_version.setter
+    def min_version(self, value):
+        self._min_version = value
+        if self._min_version is None:
+            self._min_version_parsed = parse_version('-999.-999.-999')
+        else:
+            self._min_version_parsed = parse_version(self._min_version)
+
+    @property
+    def exclude(self):
+        return self._exclude
+
+    @exclude.setter
+    def exclude(self, value):
+        if value is None:
+            value = []
+        elif not isinstance(value, (list, tuple)):
+            value = [value]
+        value = list(value)
+        self._exclude = value
+
     def uri_exists(self, index_url=None):
         """Return if the given URL/URI exists."""
         return uri_exists(index_url or self.index_url)
@@ -68,7 +107,8 @@ class HttpListVersions(HTMLParser):
         if tag == 'a' and (len(self.extensions) == 0 or any(ext in href for ext in self.extensions)):
             href = urljoin(self.index_url, href)
             name, version = get_name_version(href)
-            if (name, version) not in self.saved_data:
+            if (version not in self.exclude and parse_version(version) >= self._min_version_parsed and
+                    (name, version) not in self.saved_data):
                 self.saved_data[(name, version)] = href
             # print(name, version, href)
 
@@ -87,18 +127,21 @@ class HttpListVersions(HTMLParser):
         return filename
 
     @classmethod
-    def get_versions(cls, package, index_url='https://pypi.org/simple/', extensions=None, **kwargs):
+    def get_versions(cls, package, index_url='https://pypi.org/simple/', extensions=None,
+                     min_version=None, exclude=None, **kwargs):
         """Return a series of package versions.
 
         Args:
             package (str): Name of the package/library you want to ge the versions for (Example: "requests").
             index_url (str) ['https://pypi.org/simple/']: Simple url to get the package and it's versions from.
             extensions (list/str) [None]: List of allowed extensions (Example: [".whl", ".tar.gz"]).
+            min_version (str)[None]: Minimum version to allow.
+            exclude (list)[None]: List of versions that are excluded.
 
         Returns:
             data (OrderedDict): Dictionary of {(package name, version): href}
         """
-        parser = cls(index_url, extensions, **kwargs)
+        parser = cls(index_url, extensions, min_version=min_version, exclude=exclude, **kwargs)
         resp = requests.get(parser.index_url + package)
         if resp.status_code != 200:
             raise ValueError('Invalid URL.')
@@ -107,8 +150,8 @@ class HttpListVersions(HTMLParser):
         return parser.saved_data
 
     @classmethod
-    def download(cls, package, version=None, download_dir='.',
-                 index_url='https://pypi.org/simple/', extensions=None, chunk_size=1024, **kwargs):
+    def download(cls, package, version=None, download_dir='.', index_url='https://pypi.org/simple/', extensions=None,
+                 min_version=None, exclude=None, chunk_size=1024, **kwargs):
         """Return a series of package versions.
 
         Args:
@@ -117,12 +160,15 @@ class HttpListVersions(HTMLParser):
             download_dir (str)['.']: Download directory.
             index_url (str) ['https://pypi.org/simple/']: Simple url to get the package and it's versions from.
             extensions (list/str) [None]: List of allowed extensions (Example: [".whl", ".tar.gz"]).
+            min_version (str)[None]: Minimum version to allow.
+            exclude (list)[None]: List of versions that are excluded.
             chunk_size (int)[1024]: Save the file with this chunk size.
 
         Returns:
             filename (str): Filename of the downloaded file.
         """
-        versions = cls.get_versions(package, index_url=index_url, extensions=extensions, **kwargs)
+        versions = cls.get_versions(package, index_url=index_url, extensions=extensions,
+                                    min_version=min_version, exclude=exclude, **kwargs)
         if version is None:
             href = versions[list(versions)[-1]]  # Get latest version
         else:
@@ -151,8 +197,11 @@ if __name__ == '__main__':
     P.add_argument('-i', '--index_url', type=str, default='https://pypi.org/simple/', help='Index url to search.')
     P.add_argument('-e', '--extensions', metavar='N', type=str, nargs='+',
                    help='Allowed extensions (".whl", ".tar.gz")')
+    P.add_argument('--min_version', type=str, default=None, help='Minimum version number to allow')
+    P.add_argument('--exclude', metavar='N', type=str, nargs='*', help='Exclude versions')
     ARGS = P.parse_args()
 
-    VERSIONS = HttpListVersions.get_versions(ARGS.package, index_url=ARGS.index_url, extensions=ARGS.extensions)
+    VERSIONS = HttpListVersions.get_versions(ARGS.package, index_url=ARGS.index_url, extensions=ARGS.extensions,
+                                             min_version=ARGS.min_version, exclude=ARGS.exclude)
     for (N, V), HREF in VERSIONS.items():
         print(N, V, HREF)
